/*
       ************************************
       ************************************
       **         TEST MACROSHIELS       **
       ************************************
       ************************************/

#include <Wire.h>
#include <TM1638.h>
#include <larttm1638.h>
#include <Tone.h>
#include <Time.h>
#include <TimeAlarms.h>
#include <elapsedMillis.h>

/*    ******************************************************************************************************************
      ** TODO: ДЛЯ УСПЕШНОЙ СБОРКИ ПРОГРАММЫ УСТАНОВИТЕ БИБЛИОТЕКИ: Time и TimeAlarms;                                **
      **       ИСПОЛЬЗУЙТЕ ВЕРСИЮ ARDUINO IDE 1.65 И МЕНЕЖДЕР БИБЛИОТЕК (Эскиз->Include Library->Manage Libraries)    **
      ******************************************************************************************************************/

/*    *********************************
      **  НАСТРОЙКА ВЫВОДОВ АРДУИНО  **
      **     ИНДИКАТОР И КЛАВИАТУРЫ  **
      *********************************/

// ***************************************************************************
// ОПРЕДЕЛЕНИЕ ВЫВОДОВ ИНДИКАТОРА (TODO: ВПИСАТЬ НОМЕРА ВЫВОДОВ АРДУИНО)

#define  SCLK    10
#define  DATA     9
#define  STROB   11

// ВЫВОД ЗУММЕРА (TODO: ВПИСАТЬ НОМЕР ВЫВОДА АРДУИНО)
#define  BUZZER  12
// ***************************************************************************

/*    *********************************
      **  НАСТРОЙКИ ПО УМОЛЧАНИЮ     **
      *********************************/

// ***************************************************************************

/** -------------------------------------------------------
 * ВРЕМЯ ДО НАЧАЛА АВТОПОВТОРА (мс)
 */
#define KEY_DELAY   300
/** -------------------------------------------------------
 * ЧАСТОТА ОЗВУЧКИ КНОПОК ПО УМОЛЧАНИЮ (Гц)
 */
#define FREQ_KEY_SOUND   1000
/** -------------------------------------------------------
 * ВРЕМЯ ОЗВУЧКИ КНОПОК ПО УМОЛЧАНИЮ (мс)
 */
#define INT_KEY_SOUND   20
/** -------------------------------------------------------
 * ИНТЕРВАЛ АВТОПОВТОРА ПО УМОЛЧАНИЮ (мс)
 */
#define INT_AUTOREP   20
/** -------------------------------------------------------
 * ИНТЕРВАЛ МИГАНИЯ (мс)
 */
#define INT_BLINK   250

// ***************************************************************************

// ***************************************************************************
/// ОБЪЕКТ ЗУММЕРА
Tone buzzer;
/// ОБЪЕКТ ИНДИКАТОРА
LART_TM1638 module( DATA, SCLK, STROB );
// ***************************************************************************

/*    *********************************
      **      СКАН-КОДЫ КНОПОК       **
      *                               *
      *      KEY1  KEY2  KEY3         *
      *      KEY4  KEY5  KEY6         *
      *      KEY7  KEY8  KEY9         *
      *      KEYA  KEYB  KEYC         *
      *                               *
      *********************************/

// ***************************************************************************

typedef enum KEY_CODE_E
{
  KEY1 = 0x04000000,
  KEY2 = 0x02000000,
  KEY3 = 0x01000000,
  KEY4 = 0x40000000,
  KEY5 = 0x20000000,
  KEY6 = 0x10000000,
  KEY7 = 0x00040000,
  KEY8 = 0x00020000,
  KEY9 = 0x00010000,
  KEYA = 0x00100000,
  KEYB = 0x00200000,
  KEYC = 0x00400000
} keycode_t;
/** -------------------------------------------------------
 * "КОДОВАЯ ТАБЛИЦА"
 */
const keycode_t key_table[ ] =
{ KEY1 , KEY2 , KEY3 ,
  KEY4 , KEY5 , KEY6 ,
  KEY7 , KEY8 , KEY9 ,
  KEYA , KEYB , KEYC
};

#define KEY_NUMBER  ( sizeof( key_table ) / sizeof( keycode_t ) )

/** ------------------------------------------------------------
 *  КЛАСС ОБРАБОТКИ СИГНАЛОВ КЛАВИАТУРЫ
 */
class keyboard
{
  public:
    /** ------------------------------------------------------
     * @brief КОНСТРУКТОР
     */
    keyboard( ) {
      int i = KEY_NUMBER;
      while ( i-- ) {
        autoRepInterval[ i ] = 0;
        intKeySound[ i ]     = INT_KEY_SOUND;
        freqKeySound[ i ]    = FREQ_KEY_SOUND;
      };
    }

    /** -----------------------------------------------------------
     * @brief ФУНКЦИЯ ОПРОСА КНОПОК
     * @return номер нажатой кнопки или -1 если ни одна кнопка не нажата
     */
    int key_pooling( void )
    {
      // статические переменные
      static unsigned long prev_code = 0;
      static int      autoRepCounter = 0;
      // чтение кода кнопки
      unsigned long code = module.getKeys( );
      // Время до первого автоповтора
      if ( code == 0 ) autoRepCounter = KEY_DELAY;
      // получаем номер кнопки
      int res  = name_to_number( prev_code );
      // счет времени до автоповтора
      if ( autoRepCounter > 0 ) {
        --autoRepCounter;
      }
      // Если отсчет времени после нажатия завершился и код кнопки валиден и задан период автоповтора
      if ( ( autoRepCounter <= 0 ) && ( autoRepInterval[ res ] != 0 )  )
      {
        autoRepCounter = autoRepInterval[ res ]; // Устанавливаем значение автоповтора
        code = 0;     // имитируем приход заднего фронта
      }
      // фиксация заднего фронта
      if ( prev_code == code )  res = -1;
      prev_code = code;
      if ( res >= 0 && intKeySound[ res ] >= 0 && freqKeySound[ res ] >= 0 )
        buzzer.play( freqKeySound[ res ] , intKeySound[ res ] );
      return res;
    }
    /** ----------------------------------------------------
     * Функция установки озвучки кнопок
     * @param key - номер кнопки
     * @param freq - частота озвучки в Гц (если <= 0 звук отключен )
     * @param interval - интервал озвучки в мс (если <= 0 звук отключен )
     */
    void setKeySound( int key , int freq , int interval )
    {
      if ( key < 0 ) key = 0;
      if ( key > ( KEY_NUMBER - 1 ) ) key = KEY_NUMBER - 1;
      if ( interval < 0 ) interval = 0;
      if ( freq < 0 ) freq = 0;
      intKeySound[ key ] = interval;
      freqKeySound[ key ]  = freq;
    }
    /** ----------------------------------------------------
     * Функция установки интервала автоповтора
     * @param key - номер кнопки
     * @param interval - интервал автоповтора в мс (если <= 0 автоповтора нет )
     */
    void setAutoRepeate( int key , int interval )
    {
      if ( key < 0 ) key = 0;
      if ( key > ( KEY_NUMBER - 1 ) ) key = KEY_NUMBER - 1;
      if ( interval < 0 ) interval = 0;
      autoRepInterval[ key ] = interval;
    }
    /** ----------------------------------------------------
     * Функция преобразования сканкода кнопки в ее порядковый номер в таблице key_table
     * @param key - сканкода
     * @return номер кнопки от 0 до KEY_NUMBER-1 или -1 в случае ошибки
     */
    int name_to_number( unsigned long key )
    {
      int res = -1;
      for ( int i = 0 ; i < KEY_NUMBER ; i++ )
        if ( key == key_table[ i ] ) {
          res = i;
          break;
        }
      return res;
    }

  private:

    int autoRepInterval[ KEY_NUMBER ];
    int freqKeySound[ KEY_NUMBER ];
    int intKeySound[ KEY_NUMBER ];
};


/** --------------------------------------------------------------------------------------------------------------
 * НОТНЫЙ СТАН
 */
#define OCTAVE_OFFSET 0
int notes[] = { 0,
NOTE_C4, NOTE_CS4, NOTE_D4, NOTE_DS4, NOTE_E4, NOTE_F4, NOTE_FS4, NOTE_G4, NOTE_GS4, NOTE_A4, NOTE_AS4, NOTE_B4,
NOTE_C5, NOTE_CS5, NOTE_D5, NOTE_DS5, NOTE_E5, NOTE_F5, NOTE_FS5, NOTE_G5, NOTE_GS5, NOTE_A5, NOTE_AS5, NOTE_B5,
NOTE_C6, NOTE_CS6, NOTE_D6, NOTE_DS6, NOTE_E6, NOTE_F6, NOTE_FS6, NOTE_G6, NOTE_GS6, NOTE_A6, NOTE_AS6, NOTE_B6,
NOTE_C7, NOTE_CS7, NOTE_D7, NOTE_DS7, NOTE_E7, NOTE_F7, NOTE_FS7, NOTE_G7, NOTE_GS7, NOTE_A7, NOTE_AS7, NOTE_B7
};
/** ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 * МАССИВ МЕЛОДИЙ (РАССТАНОВКОЙ КОММЕНТАРИЕВ ВЫБЕРЕ ТАКОЕ, ПРИ КОТОРОМ ПРОГРАММ УМЕЩАЕТСЯ В ПАМЯТЬ ЕЩЕ 120 В РЕПОЗИТОРИИ https://github.com/granadaxronos/120-SONG_NOKIA_RTTTL_RINGTONE_PLAYER_FOR_ARDUINO_UNO/blob/master/RTTTL_PLAYER/songs.h)
 */
const char *songs[ ] = {  "The Simpsons:d=4,o=5,b=160:c.6,e6,f#6,8a6,g.6,e6,c6,8a,8f#,8f#,8f#,2g,8p,8p,8f#,8f#,8f#,8g,a#.,8c6,8c6,8c6,c6" , 
                          //"Indiana:d=4,o=5,b=250:e,8p,8f,8g,8p,1c6,8p.,d,8p,8e,1f,p.,g,8p,8a,8b,8p,1f6,p,a,8p,8b,2c6,2d6,2e6,e,8p,8f,8g,8p,1c6,p,d6,8p,8e6,1f.6,g,8p,8g,e.6,8p,d6,8p,8g,e.6,8p,d6,8p,8g,f.6,8p,e6,8p,8d6,2c6" ,
                          //"TakeOnMe:d=4,o=4,b=160:8f#5,8f#5,8f#5,8d5,8p,8b,8p,8e5,8p,8e5,8p,8e5,8g#5,8g#5,8a5,8b5,8a5,8a5,8a5,8e5,8p,8d5,8p,8f#5,8p,8f#5,8p,8f#5,8e5,8e5,8f#5,8e5,8f#5,8f#5,8f#5,8d5,8p,8b,8p,8e5,8p,8e5,8p,8e5,8g#5,8g#5,8a5,8b5,8a5,8a5,8a5,8e5,8p,8d5,8p,8f#5,8p,8f#5,8p,8f#5,8e5,8e5"
                          "Entertainer:d=4,o=5,b=140:8d,8d#,8e,c6,8e,c6,8e,2c.6,8c6,8d6,8d#6,8e6,8c6,8d6,e6,8b,d6,2c6,p,8d,8d#,8e,c6,8e,c6,8e,2c.6,8p,8a,8g,8f#,8a,8c6,e6,8d6,8c6,8a,2d6",
                          //"Muppets:d=4,o=5,b=250:c6,c6,a,b,8a,b,g,p,c6,c6,a,8b,8a,8p,g.,p,e,e,g,f,8e,f,8c6,8c,8d,e,8e,8e,8p,8e,g,2p,c6,c6,a,b,8a,b,g,p,c6,c6,a,8b,a,g.,p,e,e,g,f,8e,f,8c6,8c,8d,e,8e,d,8d,c",
                          "Xfiles:d=4,o=5,b=125:e,b,a,b,d6,2b.,1p,e,b,a,b,e6,2b.,1p,g6,f#6,e6,d6,e6,2b.,1p,g6,f#6,e6,d6,f#6,2b.,1p,e,b,a,b,d6,2b.,1p,e,b,a,b,e6,2b.,1p,e6,2b.",
                          //"Looney:d=4,o=5,b=140:32p,c6,8f6,8e6,8d6,8c6,a.,8c6,8f6,8e6,8d6,8d#6,e.6,8e6,8e6,8c6,8d6,8c6,8e6,8c6,8d6,8a,8c6,8g,8a#,8a,8f",
                          //"20thCenFox:d=16,o=5,b=140:b,8p,b,b,2b,p,c6,32p,b,32p,c6,32p,b,32p,c6,32p,b,8p,b,b,b,32p,b,32p,b,32p,b,32p,b,32p,b,32p,b,32p,g#,32p,a,32p,b,8p,b,b,2b,4p,8e,8g#,8b,1c#6,8f#,8a,8c#6,1e6,8a,8c#6,8e6,1e6,8b,8g#,8a,2b",
                          "Bond:d=4,o=5,b=80:32p,16c#6,32d#6,32d#6,16d#6,8d#6,16c#6,16c#6,16c#6,16c#6,32e6,32e6,16e6,8e6,16d#6,16d#6,16d#6,16c#6,32d#6,32d#6,16d#6,8d#6,16c#6,16c#6,16c#6,16c#6,32e6,32e6,16e6,8e6,16d#6,16d6,16c#6,16c#7,c.7,16g#6,16f#6,g#.6",
                          //"MASH:d=8,o=5,b=140:4a,4g,f#,g,p,f#,p,g,p,f#,p,2e.,p,f#,e,4f#,e,f#,p,e,p,4d.,p,f#,4e,d,e,p,d,p,e,p,d,p,2c#.,p,d,c#,4d,c#,d,p,e,p,4f#,p,a,p,4b,a,b,p,a,p,b,p,2a.,4p,a,b,a,4b,a,b,p,2a.,a,4f#,a,b,p,d6,p,4e.6,d6,b,p,a,p,2b",
                          "StarWars:d=4,o=5,b=45:32p,32f#,32f#,32f#,8b.,8f#.6,32e6,32d#6,32c#6,8b.6,16f#.6,32e6,32d#6,32c#6,8b.6,16f#.6,32e6,32d#6,32e6,8c#.6,32f#,32f#,32f#,8b.,8f#.6,32e6,32d#6,32c#6,8b.6,16f#.6,32e6,32d#6,32c#6,8b.6,16f#.6,32e6,32d#6,32e6,8c#6",
                          //"GoodBad:d=4,o=5,b=56:32p,32a#,32d#6,32a#,32d#6,8a#.,16f#.,16g#.,d#,32a#,32d#6,32a#,32d#6,8a#.,16f#.,16g#.,c#6,32a#,32d#6,32a#,32d#6,8a#.,16f#.,32f.,32d#.,c#,32a#,32d#6,32a#,32d#6,8a#.,16g#.,d#",
                          //"TopGun:d=4,o=4,b=31:32p,16c#,16g#,16g#,32f#,32f,32f#,32f,16d#,16d#,32c#,32d#,16f,32d#,32f,16f#,32f,32c#,16f,d#,16c#,16g#,16g#,32f#,32f,32f#,32f,16d#,16d#,32c#,32d#,16f,32d#,32f,16f#,32f,32c#,g#",
                          //"A-Team:d=8,o=5,b=125:4d#6,a#,2d#6,16p,g#,4a#,4d#.,p,16g,16a#,d#6,a#,f6,2d#6,16p,c#.6,16c6,16a#,g#.,2a#",
                          //"Flinstones:d=4,o=5,b=40:32p,16f6,16a#,16a#6,32g6,16f6,16a#.,16f6,32d#6,32d6,32d6,32d#6,32f6,16a#,16c6,d6,16f6,16a#.,16a#6,32g6,16f6,16a#.,32f6,32f6,32d#6,32d6,32d6,32d#6,32f6,16a#,16c6,a#,16a6,16d.6,16a#6,32a6,32a6,32g6,32f#6,32a6,8g6,16g6,16c.6,32a6,32a6,32g6,32g6,32f6,32e6,32g6,8f6,16f6,16a#.,16a#6,32g6,16f6,16a#.,16f6,32d#6,32d6,32d6,32d#6,32f6,16a#,16c.6,32d6,32d#6,32f6,16a#,16c.6,32d6,32d#6,32f6,16a#6,16c7,8a#.6",
                          //"Jeopardy:d=4,o=6,b=125:c,f,c,f5,c,f,2c,c,f,c,f,a.,8g,8f,8e,8d,8c#,c,f,c,f5,c,f,2c,f.,8d,c,a#5,a5,g5,f5,p,d#,g#,d#,g#5,d#,g#,2d#,d#,g#,d#,g#,c.7,8a#,8g#,8g,8f,8e,d#,g#,d#,g#5,d#,g#,2d#,g#.,8f,d#,c#,c,p,a#5,p,g#.5,d#,g#",
                          //"Gadget:d=16,o=5,b=50:32d#,32f,32f#,32g#,a#,f#,a,f,g#,f#,32d#,32f,32f#,32g#,a#,d#6,4d6,32d#,32f,32f#,32g#,a#,f#,a,f,g#,f#,8d#",
                          //"MahnaMahna:d=16,o=6,b=125:c#,c.,b5,8a#.5,8f.,4g#,a#,g.,4d#,8p,c#,c.,b5,8a#.5,8f.,g#.,8a#.,4g,8p,c#,c.,b5,8a#.5,8f.,4g#,f,g.,8d#.,f,g.,8d#.,f,8g,8d#.,f,8g,d#,8c,a#5,8d#.,8d#.,4d#,8d#.",
                          //"LeisureSuit:d=16,o=6,b=56:f.5,f#.5,g.5,g#5,32a#5,f5,g#.5,a#.5,32f5,g#5,32a#5,g#5,8c#.,a#5,32c#,a5,a#.5,c#.,32a5,a#5,32c#,d#,8e,c#.,f.,f.,f.,f.,f,32e,d#,8d,a#.5,e,32f,e,32f,c#,d#.,c#",
                          //"MissionImp:d=16,o=6,b=95:32d,32d#,32d,32d#,32d,32d#,32d,32d#,32d,32d,32d#,32e,32f,32f#,32g,g,8p,g,8p,a#,p,c7,p,g,8p,g,8p,f,p,f#,p,g,8p,g,8p,a#,p,c7,p,g,8p,g,8p,f,p,f#,p,a#,g,2d,32p,a#,g,2c#,32p,a#,g,2c,a#5,8c,2p,32p,a#5,g5,2f#,32p,a#5,g5,2f,32p,a#5,g5,2e,d#,8d" 
                        };


/** -----------------------------------------------------------------------
 * КЛАСС ВОСПРОИЗВЕДЕНИЕ МЕЛОДИЙ (СМ. ПРИМЕР К Tone.h RTTTL)
 */
class melody
{
  public:

    void play( const char *p )
    {
      position = ( char* )p;
      f_play = true;
      // format: d=N,o=N,b=NNN:
      // find the start (skip name, etc)
      while (*p != ':') p++;   // ignore name
      p++;                     // skip ':'

      // get default duration
      if (*p == 'd')
      {
        p++; p++;              // skip "d="
        num = 0;
        while (isdigit(*p))
        {
          num = (num * 10) + (*p++ - '0');
        }
        if (num > 0) default_dur = num;
        p++;                   // skip comma
      }
            // get default octave
      if (*p == 'o')
      {
        p++; p++;              // skip "o="
        num = *p++ - '0';
        if (num >= 3 && num <= 7) default_oct = num;
        p++;                   // skip comma
      }
      // get BPM
      if (*p == 'b')
      {
        p++; p++;              // skip "b="
        num = 0;
        while (isdigit(*p))
        {
          num = (num * 10) + (*p++ - '0');
        }
        bpm = num;
        p++;                   // skip colon
      }
            // BPM usually expresses the number of quarter notes per minute
      wholenote = (60 * 1000L / bpm) * 4;  // this is the time for whole note (in milliseconds)
    };
    
    void stop( ) {
      position = 0;
    };
    
    
    bool isPlay( ) {
      return f_play;
    };

    void play_rtttl(  )
    {
      if ( !f_play ) {
        default_dur = 4;
        default_oct = 6;
        bpm = 63;
        return;
      }
      if( timeElapsed < duration ) return;

      // now begin note loop
      if (*position)
      {
        // first, get note duration, if available
        num = 0;
        while (isdigit(*position))
        {
          num = (num * 10) + (*position++ - '0');
        }

        if (num) duration = wholenote / num;
        else duration = wholenote / default_dur;  // we will need to check if we are a dotted note after

        // now get the note
        note = 0;

        switch (*position)
        {
          case 'c':
            note = 1;
            break;
          case 'd':
            note = 3;
            break;
          case 'e':
            note = 5;
            break;
          case 'f':
            note = 6;
            break;
          case 'g':
            note = 8;
            break;
          case 'a':
            note = 10;
            break;
          case 'b':
            note = 12;
            break;
          case 'p':
          default:
            note = 0;
        }
        position++;

        // now, get optional '#' sharp
        if (*position == '#')
        {
          note++;
          position++;
        }

        // now, get optional '.' dotted note
        if (*position == '.')
        {
          duration += duration / 2;
          position++;
        }

        // now, get scale
        if (isdigit(*position))
        {
          scale = *position - '0';
          position++;
        }
        else
        {
          scale = default_oct;
        }

        scale += OCTAVE_OFFSET;

        if (*position == ',')
          position++;       // skip comma for next note (or we may be at the end)

        // now play the note

        if (note)
        {
          buzzer.play(notes[(scale - 4) * 12 + note] , duration );
          timeElapsed = 0;
        }
        else
        {
          timeElapsed = 0;
        }
      }
      else
        f_play = false;
    }
  private:
    bool f_play;
    char *position;

          // Absolutely no error checking in here
      byte default_dur = 4;
      byte default_oct = 6;
      int bpm = 63;
      int num;
      long wholenote;
      long duration;
      byte note;
      byte scale;
      elapsedMillis timeElapsed;
};

/*    *********************************
      **      НАЗНАЧЕНИЯ КНОПОК      **
      *                               *
      *      ЧАС+  МИН+  СЕК0         *
      *      ЧАС-  МИН-  STOP         *
      *       M1    M2    M3          *
      *       M4    M5   STOP         *
      *                               *
      *           ИНДИКАТОР           *
      *           ЧЧ-ММ-СС            *
      *********************************/




/// ---------------------------------------------------------------------
/// ОБЪЕКТ Клавиатуры
keyboard keys;

/// ---------------------------------------------------------------------
/// ОБЪЕКТ Плеера
melody m_play;

/// ---------------------------------------------------------------------
/// СТАРТОВЫЕ НАСТРОЙКИ
void setup()
{
  setTime( 0 , 0 , 0 , 1 , 1 , 1 );

  buzzer.begin( BUZZER );
  Serial.begin( 115200 );
  // --------------------------------------
  // Приветсвие при старте
  // --------------------------------------
  module.setDisplayString( "CONTEST!" );
  for ( int i = 0; i < 50 ; i++ )
  {
    buzzer.play( 200 + i * 20 , 20 );
    delay( 20 );
  }
  // --------------------------------------
  keys.setAutoRepeate( 0 , INT_AUTOREP );
  keys.setAutoRepeate( 1 , INT_AUTOREP );

  keys.setAutoRepeate( 3 , INT_AUTOREP );
  keys.setAutoRepeate( 4 , INT_AUTOREP );
}

/// ---------------------------------------------------------------------
// Фаза мигания
bool blink_state;
// Счетчик мс до следующего обновления индикатора
unsigned int update_cnt;

/// ---------------------------------------------------------------------
/// РАСПЕЧАТКА ВРЕМЕНИ
void print_time(  )
{
  char str[ 16 ];
  sprintf( str , "%02d %02d %02d" , hour( ) , minute( ) , second( ) );
  if ( blink_state ) {
    str[ 2 ] = '-';
    str[ 2 + 3 ] = '-';
  }
  module.setDisplayString( str );
}

/// ---------------------------------------------------------------------
/// СИГНАЛ КАЖДЫЙ ЧАС
void EachHour( void )
{
  buzzer.play( FREQ_KEY_SOUND * 2 , 100 );
}



/// ---------------------------------------------------------------------
/// ГЛАВНЫЙ ЦИКЛ
void loop()
{
  // --------------------------------------------------------------------------
  // Опрос кнопок
  int keyCode =  keys.key_pooling( );
  if ( keyCode >= 0 )
  {
    int adjust = 0;
    // Разбор кнопок
    switch ( keyCode )
    {
      // установка часов
      case 0: adjust = 3600;
        break;
      case 3: adjust = -3600;
        break;
      // установка минут
      case 1: adjust = 60;
        break;
      case 4: adjust = -60;
        break;
      // сброс секунд
      case 2: adjust = -second( );
        break;
      case 5:
        m_play.stop( );
        break;
      case 6:
        m_play.play( songs[ 0 ] );
        break;
      case 7:
        m_play.play( songs[ 1 ] );
        break;
      case 8:
        m_play.play( songs[ 2 ] );
        break; 
      case 9:
        m_play.play( songs[ 3 ] );
        break; 
      case 10:
        m_play.play( songs[ 4 ] );
        break; 
      case 11:
        m_play.stop();
        break; 
    }
    adjustTime( adjust );
    // распечатка времени
    print_time( );

  }
  // --------------------------------------------------------------------------
  // Обновление индикатора: раз в INT_BLINK мс
  if ( ++update_cnt > INT_BLINK )
  {
    // сброс счетчика мс и инверсия состояния мигания
    update_cnt = 0;
    blink_state ^= 1;
    // распечатка времени
    print_time( );
  }

  // Проверка на окончание часа
  if ( minute() == 0 && second() == 0 ) EachHour( );
  // Чтение нот плеером
  m_play.play_rtttl( );
  delay( 1 ); 
}/// -----------------------------------------------------------------------



